# airPressureEstimate for the function
#directory = '/Users/gfivash/Desktop/JvB Robot/Wave-loggers/Output/Greg wavelogger #49'
#burstFile = 'bursts.csv'

# Choose wave-logger folder
#setwd(directory)

# Load pressure data & set threshold quantile
#pressureData_BAR = read.csv(burstFile, header = F)
#largeIncreaseFilter = 0.05

# -----
# Function begins
airPressureCorrection = function(pressureData_Pa, largeIncreaseFilter, largeValueFilter){
  
  # Average pressure readings within each burst
  meanPressure_Pa = vector()
  for(i in 1:dim(pressureData_Pa)[2]){
    meanPressure_Pa[i] = mean(pressureData_Pa[is.na(pressureData_Pa[,i]) == F,i]) 
  }
  
  # Take the limit (derivative) of the averaged total pressure values:
    # This yields the slope values so that you can detect fast/slow changes in pressure
    # Comparitively fast pressure changes indicate submergence in water while
    # slow changes indicate the more gradual shifts in air pressure.
    # We can use these slope values to disentangle pressure changes due to water or air
  
  x = 1:(length(meanPressure_Pa) - 1)
  y = meanPressure_Pa
  slope = abs((y[x] - y[x + 1])/1)
  
  # Define a slope value threshold, over which all
  # high slope values (representing submerged moments) are removed
  # and apply to mean pressure data

  threshold = quantile(slope, largeIncreaseFilter, na.rm = T)
  airPressureEstimate = meanPressure_Pa
  airPressureEstimate[(slope[1:length(airPressureEstimate)] > threshold) 
                      | (airPressureEstimate >= quantile(meanPressure_Pa,largeValueFilter, na.rm = T))] = NA
  
  # Interpolate NA values between known air pressure values by drawing a straight line
  # between the known values and determine the intermediate values by the slope formula:
  # Change in pressure / Change in time (Rise/Run!)
    
  numericCount = cumsum(is.na(airPressureEstimate) == F)
  numericAirPressureEstimate = airPressureEstimate[is.na(airPressureEstimate) == F]
  output = airPressureEstimate
  rise = vector()
  run = vector()
  runforward = vector()
  runback = vector()
  riseOverRun = vector()
  
  for (i in 1:length(airPressureEstimate)){
  
    # Calculate run
    # Corner cases
    if((numericCount[i] == 0) | (numericCount[i] == max(numericCount))){
      run[i] = NA
      rise[i] = NA
      
      # Regular cases where vector positions bound either side of our targeted NA position
    } else if (is.na(airPressureEstimate[i]) == T){
      
      # Run forward until a numeric is found
      runforward[i] = 0
      ii = 0
      while(is.na(airPressureEstimate[i + ii]) == T){
        runforward[i] = runforward[i] + as.numeric(is.na(airPressureEstimate[i + ii]))
        ii = ii + 1
      }
      
      # Run backward until a numeric is found
      runback[i] = 0
      ii = 0
      while(is.na(airPressureEstimate[i - ii]) == T){
        runback[i] = runback[i] + as.numeric(is.na(airPressureEstimate[i - ii]))
        ii = ii + 1
      }
      
      # Combine runforward & backward to make total run
      run[i] = runforward[i] + runback[i]
      
      # Calculate rise
      rise[i] = numericAirPressureEstimate[numericCount[i] + 1] - numericAirPressureEstimate[numericCount[i]]
    }
    
    # Calculate rise/run
    riseOverRun[i] = rise[i]/run[i] 
    
    # Interpolate output from airPressureEstimate & rise/run
    if(is.na(airPressureEstimate[i]) == T){
      
      # Corner cases
      if(numericCount[i] == 0){
        output[i] = numericAirPressureEstimate[1]
      } else if (numericCount[i] == max(numericCount)){
        output[i] = numericAirPressureEstimate[length(numericAirPressureEstimate)]
        
        # Cummulate rise/run & combine with known airPressureEstimate to calculate output  
      } else {
        output[i] = output[i - 1] + rise[i]/run[i]
      }
      
    }
  }
  meanAirPressureEstimate_Pa = output
  
  # Correct pressure data so that fluctuating air pressure is removed. 
  # It now represents water pressure only. (Also, values below zero become equal to zero)
  pressureWater_Pa = array(0, dim = dim(pressureData_Pa))
  for (i in 1:dim(pressureData_Pa)[2]){
    pressureWater_Pa[,i] = pressureData_Pa[,i] - meanAirPressureEstimate_Pa[i]
    pressureWater_Pa[pressureWater_Pa[,i] < 0,i] == 0
  }
  
  # Calculate average water pressure per burst (for test plotting)
  meanWaterPressure_Pa = vector()
  for(i in 1:dim(pressureWater_Pa)[2]){
    meanWaterPressure_Pa[i] = mean(pressureWater_Pa[is.na(pressureWater_Pa[,i]) == F,i]) 
  }
  
  # Test plot: Use this plot to find best 'thresholdQuantile' value
  # Attempt to make red 'air pressure' line fit low tide line
  par(mfrow = c(2,1))
  plot(meanPressure_Pa, type = 'l', ylim = c(0, max(meanPressure_Pa, na.rm = T)),
       ylab = 'Pressure (Pa)', xlab = 'Burst number', main = 'Total pressure')
  lines(meanAirPressureEstimate_Pa, type = 'l', col = 2)
  legend('topleft', legend = 'Air pressure estimation', lty = 1, col = 2, bty = 'n', cex = 0.7)
  plot(meanWaterPressure_Pa, type = 'l', col = 4,
       ylab = 'Pressure (Pa)', xlab = 'Burst number', main = 'Water pressure',
       ylim = c(min(meanWaterPressure_Pa, na.rm = T), max(meanPressure_Pa, na.rm = T)))
  lines(x = c(0,length(meanWaterPressure_Pa)),y = c(0,0), type = 'l', col = 2)
  legend('topleft', legend = 'Baseline (zero)', lty = 1, col = 2, bty = 'n', cex = 0.7)
  par(mfrow = c(1,1))
  
  outputList_airCorrection = list(meanWaterPressure_Pa, pressureWater_Pa)
  return(outputList_airCorrection)
}
