### Calculating Inundation Frequency (Prop.) ###

# Function begins
inundFreqFunction = function(meanWaterPressure.Pa, timeStamp.bursts, burstInterval.sec){
  
  ## Threshold (manually set)
  # Vary this value until estimated emerged/submerged fit is strong (View in output figures: Inundation frequency)
  # Value range: n > 0
  minimumSubmerged.m = 0.01
  
  # Physical parameters
  g = 9.81          # gravitational acceleration (m/s^2)
  rho = 1024       # water density (kg/m^3)
  
  # Subsetting dataset to include only emergent or submergent values
  minimumSubmergedPa = minimumSubmerged.m * g * rho
  pressureQuantileSeq = vector()
  quantileSeq = seq(0, 1, length.out = 100)
  for (i in 1:length(quantileSeq)){
    pressureQuantileSeq[i] = quantile(meanWaterPressure.Pa, quantileSeq[i], na.rm = T)
  }
  thresholdPa = min(pressureQuantileSeq[ pressureQuantileSeq > minimumSubmergedPa])
  
  # Define submerged periods
  meanWaterPressureSubmerged.Pa = meanWaterPressure.Pa
  meanWaterPressureSubmerged.Pa[meanWaterPressure.Pa < thresholdPa] = NA
  submerged = meanWaterPressure.Pa >= thresholdPa
  
  # Define emerged periods
  meanWaterPressureEmerged.Pa = meanWaterPressure.Pa
  meanWaterPressureEmerged.Pa[meanWaterPressure.Pa >= thresholdPa] = NA
  emerged = meanWaterPressure.Pa < thresholdPa
  
  # Define installed period
  firstInstalledMeasurementFilter = 0.005
  lastInstalledMeasurementFilter = 0.995
  firstInstalledBurst = round(quantile(which(is.na(meanWaterPressureSubmerged.Pa) == F),firstInstalledMeasurementFilter, na.rm = T))
  lastInstalledBurst = round(quantile(which(is.na(meanWaterPressureSubmerged.Pa) == F),lastInstalledMeasurementFilter, na.rm = T))
  
  burstInstalled = c(firstInstalledBurst:lastInstalledBurst)
  meanWaterPressureInstalled.Pa = meanWaterPressure.Pa[burstInstalled]
  submergedInstalled = submerged[burstInstalled]
  
  ## Test plot: Use this to ensure that inundation frequency is dependable
  ## Attempt to make all flat areas appear in bottom figure and all pointy areas appear in the top figure.
    par(mfrow = c(3,1))
    plot(meanWaterPressureSubmerged.Pa,
       ylab = 'Pressure (Pa)', xlab = 'Burst number',
       main = 'Submerged',
       type = 'l', col = 4, ylim = c(0, max(meanWaterPressureInstalled.Pa, na.rm = T)))
    plot(meanWaterPressureEmerged.Pa,
       ylab = 'Pressure (Pa)', xlab = 'Burst number',
       main = 'Emerged',
       type = 'l', col = 4, ylim = c(0, max(meanWaterPressureInstalled.Pa, na.rm = T)))
    plot(meanWaterPressureInstalled.Pa,
         ylab = 'Pressure (Pa)', xlab = 'Burst number',
         main = 'Installed',
         type = 'l', col = 4, ylim = c(0, max(meanWaterPressureInstalled.Pa, na.rm = T)))
    par(mfrow = c(1,1))
    
  inundFreq = length(meanWaterPressureInstalled.Pa[meanWaterPressureInstalled.Pa >= thresholdPa])/length(meanWaterPressureInstalled.Pa)
  
  # Determine water depth
  P.Pa = pressureWater.Pa[,burstInstalled]#[,submergedInstalled]
  timeStamp.bursts = timeStamp.bursts[burstInstalled]#[submergedInstalled]
  waterDepth.m = P.Pa/(g*rho)
  waterDepth.m[waterDepth.m < 0] = 0
  
  outputList = list(inundFreq, waterDepth.m, timeStamp.bursts)
  return(outputList)
}