library(matrixStats)
library(stringr)
# --- OSSI PRESSURE TRANSDUCER DATA TO WAVE CHARACTERISTICS TRANSFORMATION SCRIPT ---

rm(list = ls())

# --- USER INPUT ---
# User Input: File directories
rootDirectory = 'G://' # insert here the link to your data folder
functionDirectory = 'G://'# insert here the link to the function folder
WLOGFilePrefix = 'WLOG'
n_waveLoggers = length(dir(rootDirectory))

# Important user input!
installationHeight.m = 0.1    # height of OSSI pressure sensitive disk above sediment surface (m)

# Vary these 2 values until estimated air pressure fit is strong (View in output figures: Air pressure correction)
# Value range: 0 - 1 (proportion values)
largeIncreaseFilter = 0.05 ##change these two values inside the loop! (script lines 90 -91)
largeValueFilter = 0.8

# ------
# SCRIPT 
# ------

# Autorun loop for analyzing multiple wave-loggers -----
# Turn on while loop for multiple loggers:
waveLogger = 1
for(waveLogger in 1:n_waveLoggers){
  inputDirectory = paste(rootDirectory, dir(rootDirectory)[waveLogger], sep = '')
  setwd(inputDirectory)
  
  # Load pressure gauge measurement settings & timestamp
  # ------------------------------------------------------
  fileList = dir(inputDirectory)[grepl('WLOG',dir(inputDirectory))]
  p = read.csv(fileList[1], nrows = 1,  header = F)
  samplingFrequency.Hz = as.numeric(unlist(strsplit(as.character(p[1,7]), split = 'F'))[2])
  samplingPeriod.sec = as.numeric(unlist(strsplit(as.character(p[1,8]), split = 'L'))[2])*60
  burstInterval.sec = as.numeric(unlist(strsplit(as.character(p[1,9]), split = 'I'))[2])*60
  
  n_measurements = samplingFrequency.Hz*samplingPeriod.sec
  n_bursts = (length(fileList))*24*60*60/burstInterval.sec
  timeStamp.bursts = read.csv('timeStamp.csv')$dateTime
  
  # Load pressure data file 'bursts.csv'
  # Convert logger's BAR units to Pa (pascal)
  setwd(inputDirectory)
  pressure.Pa = matrix(scan('pressure.csv', skip = 1, sep = ',', quiet = T), 
                       nrow = n_measurements, byrow = T) * 100000
  
  # ------
  # Air pressure correction
  # ------
  
  # Function: Air Pressure Correction
  # This function removes the pressure fluctuations occurring due to changes in air pressure
  # It returns 3 quantities: 
  # (1) the averaged-per-burst air pressure correction
  # (2) the averaged-per-burst water pressure
  # (3) the complete pressure dataset corrected to include only water pressure
  # This is used for inundation, water depth, & wave analysis
  # Run this section in case of large changes in air pressur (cuased by barnacles or algae usually)
  intervalLength.hr = 48
  tideInterval = (intervalLength.hr*60*60/burstInterval.sec)
  n_tides = floor(dim(pressure.Pa)[2]/tideInterval)
  
  # meanWaterPressure.Pa = c()
  # pressureWater.Pa = c()
  # for(i in 1:n_tides){
  #   
  #   start = 1 + tideInterval*(i - 1)
  #   end = tideInterval*i
  #   
  #   H = pressure.Pa[,start:end]
  #   
  #   setwd(functionDirectory)
  #   source("airPressureCorrectionFunction.R")
  #   outputList_airCorrection = airPressureCorrection(H, largeIncreaseFilter, largeValueFilter)
  #   
  #   meanWaterPressure.Pa = c(meanWaterPressure.Pa, outputList_airCorrection[[1]])
  #   pressureWater.Pa = cbind(pressureWater.Pa, outputList_airCorrection[[2]])
  #   
  #   
  #   par(mfrow = c(2,1))
  #   plot(colMeans(pressure.Pa)[1:end], type = 'l')
  #   plot(meanWaterPressure.Pa, type = 'l')
  #   par(mfrow = c(1,1))
  # }
  
  largeIncreaseFilter = 0.7
  largeValueFilter = 0.82
  
  setwd(functionDirectory)
  source("airPressureCorrectionFunction.R")
  outputList_airCorrection = airPressureCorrection(pressure.Pa, largeIncreaseFilter, largeValueFilter)

  meanWaterPressure.Pa = outputList_airCorrection[[1]]
  pressureWater.Pa = outputList_airCorrection[[2]]

  # ------
  # Inundation frequency determination 
  # ------
  
  # Function: Inundation frequency
  # This function separates submerged and emerged periods to calculate inundation frequency
  # It returns 3 quantities: 
  # (1) inundation frequency as a proportion of total deployment period
  # (2) the averaged-per-burst water pressure during submerged periods
  # (3) the averaged-per-burst water pressure during emerged periods
  
  setwd(functionDirectory)
  source("inundationFrequencyEstimationFunction.R") 
  outputList_InundFreq = inundFreqFunction(meanWaterPressure.Pa, timeStamp.bursts, burstInterval.sec)
  
  inundFreq = outputList_InundFreq[[1]]
  waterDepth.m = outputList_InundFreq[[2]]
  timeStamp.bursts = outputList_InundFreq[[3]]
  
  # Perform spectral analysis on water depth time series
  # ------------------------------------------------------
  
  n_measurements = dim(waterDepth.m)[1]
  n_bursts = dim(waterDepth.m)[2]
  
  # Physical parameters
  g = 9.81          # gravitational acceleration (m/s^2)
  rho = 1024       # water density (kg/m^3)
  
  meanWaterDepth.m = colMeans(waterDepth.m)
  time.s = (1:dim(waterDepth.m)[1])/samplingFrequency.Hz
  
  peakWavePeriod.sec = rep(NA, times = n_bursts)
  totalEnergy.J_m2 = rep(NA, times = n_bursts)
  Hm0.m = rep(NA, times = n_bursts)
  
  i_burst = 1
  for (i_burst in 1:n_bursts){
    
    # Detrend
    H.m = as.numeric(lm(waterDepth.m[,i_burst] ~ time.s)$residuals)
    
    # Spectral analysis (moving window)
    windowSize = 1000
    interval = windowSize/2
    
    frequency = seq(0, samplingFrequency.Hz/2, length.out = windowSize/2 + 1)[-1]
    amplitude = array(NA, dim = c((n_measurements - windowSize)/interval, length(frequency)))
    for(i_win in 1:dim(amplitude)[1]){
      start = 1 + interval*(i_win - 1)
      end = windowSize + interval*(i_win - 1)
      amplitude[i_win,] = (2*abs(fft(H.m[start:end])/windowSize))[1:(windowSize/2)]
    }
    amplitude = colMeans(amplitude)
    # plot(amplitude ~ frequency, log = '', type = 'l',ylim = c(0.000001,0.01))
    
    # Calculate signal attenuation correction at depth
    setwd(functionDirectory)
    source('depthCorrection.R')
    depthCorrection = depthCorrection(frequency, meanWaterDepth.m[i_burst], installationHeight.m)
    amplitude = amplitude / depthCorrection
    
    # (Optional) Remove infragravity waves
    amplitude[frequency < 0.05] = 0
    # (Optional) Remove turbulence
    amplitude[frequency > 2] = 0
    
    # Wave parameters
    #frequencyStepSize = mean(diff(frequency))
    totalEnergy.J_m2[i_burst] = sum(0.5*amplitude^2)*g*rho
    peakWavePeriod.sec[i_burst] = 1/frequency[which.max(amplitude[frequency >= 0.5])]
    Hm0.m[i_burst] = 4*sd(H.m)
    
    print(paste(i_burst,'/',n_bursts))
  }
  # Calculate wave parameters
  significantWaveHeight.m = 4*sqrt(totalEnergy.J_m2/(g*rho))
  rmsWaveHeight.m = 2*sqrt(2)*sqrt(totalEnergy.J_m2/(g*rho))
  orbitalVelocity.m_s2 = (significantWaveHeight.m/(2*meanWaterDepth.m))*sqrt(g*meanWaterDepth.m)
  #BSS.Pa = 0.5*rho*2.5*D50*orbitalVelocity.m_s2^2
  
  # Check on signif. wave height
  plot(significantWaveHeight.m ~ Hm0.m, log = 'yx',
       ylab = 'Hs: spectral calculation',
       xlab = 'Hs: signal SD caclulation')
  x = seq(0, 10, length.out = 1000)
  y = x
  lines(y ~ x, lwd = 2, col = 'red')
  
  par(mfrow = c(3,1))
  plot(meanWaterDepth.m, type = 'l',ylim = c(0,3))
  plot(significantWaveHeight.m, type = 'l',ylim = c(0,3))
  plot(significantWaveHeight.m ~ meanWaterDepth.m, pch = '.', 
       ylim = c(0.0000001,1), xlim = c(0.0000001,3), log = 'yx')
  par(mfrow = c(1,1))
  
  # Save everything to file
  # ------------------------------------------------------
  date = timeStamp.bursts
  wavelogger.ID = dir(rootDirectory)[waveLogger]
  
  output = data.frame(date,
                      meanWaterDepth.m,
                      significantWaveHeight.m,
                      Hm0.m,
                      rmsWaveHeight.m,
                      orbitalVelocity.m_s2,
                      totalEnergy.J_m2,
                      peakWavePeriod.sec,
                      wavelogger.ID)
  
  setwd(inputDirectory)
  write.csv(output, 
            paste('waveParameters_',wavelogger.ID,'.csv', sep = ''),
            row.names = F)
  
  print(paste(sep = '', 'Logger ', waveLogger, '/', n_waveLoggers))
}

